package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.ProductTypeEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.product.dto.CourseDTO;
import cn.rankin.data.api.product.dto.PackageDTO;
import cn.rankin.data.api.product.dto.SupportDTO;
import cn.rankin.data.api.product.entity.Course;
import cn.rankin.data.api.product.entity.Package;
import cn.rankin.data.api.product.entity.Product;
import cn.rankin.data.api.product.entity.Support;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.MerchantProductRepository;
import cn.rankin.productservice.repository.ProductRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.LinkedHashMap;
import java.util.List;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Service
public class ProductService {

    @Autowired
    private ProductRepository productRepository;

    @Autowired
    private MerchantProductRepository merchantProductRepository;

    @Autowired
    private CourseService courseService;

    @Autowired
    private SupportService supportService;

    @Autowired
    private PackageService packageService;

    public APIResult<Page<Product>> search(Product product, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = productRepository.count(product);
        Page<Product> page = new Page<>(count, pageNo, pageSize);

        Long start = page.getStart();
        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Product> productList = productRepository.find(product, start, page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(productList);

        return APIResult.ok(page);
    }

    public APIResult get(String productId) {
        Product product = productRepository.findByPid(productId);
        if (product == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        ProductTypeEnum type = product.getType();
        if (ProductTypeEnum.COURSE.equals(type)){
            return courseService.getCourse(productId);
        }else if (ProductTypeEnum.SUPPORT.equals(type)) {
            return supportService.getSupport(productId);
        }else if (ProductTypeEnum.PACKAGE.equals(type)) {
            return packageService.getPackage(productId);
        }

        return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
    }

    @Transactional
    public APIResult delete(String productId) {
        Product product = productRepository.findByPid(productId);
        if (product == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        productRepository.deleteByPid(productId);

        ProductTypeEnum type = product.getType();
        if (type.equals(ProductTypeEnum.COURSE)) {
            courseService.delete(productId);
        }else if (type.equals(ProductTypeEnum.SUPPORT)) {
            supportService.delete(productId);
        }else {
        }

        merchantProductRepository.deleteByPid(productId);
        return APIResult.ok();
    }

    @Transactional
    public APIResult<Course> create(CourseDTO courseDTO) {
        APIResult<Course> courseAPIResult = courseService.create(courseDTO);
        if (courseAPIResult.getSuccess()) {
            Product product = convert(courseAPIResult.getData());
            productRepository.save(product);
        }

        return courseAPIResult;
    }

    @Transactional
    public APIResult<Support> create(SupportDTO supportDTO) {
        APIResult<Support> supportAPIResult = supportService.create(supportDTO);
        if (supportAPIResult.getSuccess()) {
            Product product = convert(supportAPIResult.getData());
            productRepository.save(product);
        }

        return supportAPIResult;
    }

    @Transactional
    public APIResult<Package> create(PackageDTO packageDTO) {
        APIResult<Package> packageAPIResult = packageService.create(packageDTO);
        if (packageAPIResult.getSuccess()) {
            Product product = convert(packageAPIResult.getData());
            productRepository.save(product);
        }

        return packageAPIResult;
    }

    @Transactional
    public APIResult<Course> update(CourseDTO courseDTO) {
        APIResult<Course> courseAPIResult = courseService.update(courseDTO);
        if (courseAPIResult.getSuccess()) {
            Product product = convert(courseAPIResult.getData());
            productRepository.update(product);
        }

        return courseAPIResult;
    }

    @Transactional
    public APIResult<Support> update(SupportDTO supportDTO) {
        APIResult<Support> supportAPIResult = supportService.update(supportDTO);
        if (supportAPIResult.getSuccess()) {
            Product product = convert(supportAPIResult.getData());
            productRepository.update(product);
        }

        return supportAPIResult;
    }

    @Transactional
    public APIResult<Package> update(PackageDTO packageDTO) {
        APIResult<Package> packageAPIResult = packageService.update(packageDTO);
        if (packageAPIResult.getSuccess()) {
            Product product = convert(packageAPIResult.getData());
            productRepository.update(product);
        }

        return packageAPIResult;
    }
}
